# SLUB SIP Builder

## Description
A CLI tool to create valid [BagIt](https://tools.ietf.org/html/rfc8493) based submission information packages (SIPs) for ingest into the [SLUBArchiv](https://slubarchiv.slub-dresden.de/), the digital long-term preservation archive of the Saxon State and University Library Dresden (SLUB).

As a user, you have one of three options to create your SIP: `copy`, `move` or `replace`.  
NOTE: The use of option `copy` is strongly recommended. The remaining options modify your source directory. Please use these with caution!

## Support
### Supported Operating Systems
* Debian 11
* Windows 10

### Supported Versions
* official [release builds](https://git.slub-dresden.de/digital-preservation/SLUB_SIP_Builder/-/releases) only   

## Installation (Releases)

###Debian 11
1. Update system packages (optional but recommended):  
   `sudo apt update && sudo apt upgrade -y`
2. Install required tools and libraries:  
   `sudo apt install -y unzip perl libperl-dev liblwp-protocol-https-perl libfile-copy-recursive-perl libmarc-record-perl libxml-libxml-perl libxml-xpath-perl libdatetime-format-iso8601-perl cpanminus && sudo cpanm Archive::BagIt`
3. Download the latest [Release](https://git.slub-dresden.de/digital-preservation/SLUB_SIP_Builder/-/releases):  
   &rarr; ZIP file like `SLUB_SIP_Builder.version-XYZ-master.zip`
4. Extract the ZIP archive and change into the project directory:  
   `unzip SLUB_SIP_Builder.version-XYZ-master.zip`  
   `cd SLUB_SIP_Builder`
5. Run SLUB SIP Builder:  
   &rarr; Try an example command from below  
   `perl bin/slubsipbuilderbagit.pl`...

###Windows 10
1. Check UTF-8 flag at [ENG] `Systemadministration/TimeAndRegion/Region/Administration/AreaPattern` or [DE] `Systemsteuerung/Zeit und Region/Region/Verwaltung/Gebietsschema ändern/`, for using such characters as `äöüÄÖÜß`. Computer will be restarted!
2. Download strawberry perl environment [strawberry-perl-5.32.1.1-64bit-portable](https://strawberryperl.com/releases.html)
3. Extract files from archive
4. Start strawberry perl console `portableshell.bat`
5. Install additional libraries:  
   `cpanm MARC::Record XML::XPath DateTime::Format::ISO8601 Win32::Console::ANSI Archive::BagIt`
6. Download the latest [Release](https://git.slub-dresden.de/digital-preservation/SLUB_SIP_Builder/-/releases):  
   &rarr; ZIP file like `SLUB_SIP_Builder.version-XYZ-master.zip`
7. Extract the ZIP archive and change into the project directory:  
   `cd SLUB_SIP_Builder`
8. Run SLUB SIP Builder:  
   &rarr; Try an example command from below  
   `perl bin\slubsipbuilderbagit.pl`...

### HINTs

If you want a flatten script, so the lib-path is added, give the bash-script 'build_flat_perl_file.sh' a try. This script compiles the lib-path together with the bin/slubsipbuilderbagit.pl. Otherwise you must ensure to include the lib-path in your perl search paths (pE.: 'perl -I lib bin/slubsipbuilder.pl').

If your script is errorneous, try to run it with:
* `perl bin/slubsipbuilderbagit.pl --version`
* `perl bin/slubsipbuilderbagit.pl --print_sip_version`

and contact us.

(if these calls fail use the `$VERSION`  from `SLUB::LZA::SIPBuilderBagIt` module)

## Development
This guide will help you run (unreleased) development versions provided in this repository.  
Please note, only release builds are supported and will have version or release date information.

###Debian 11
1. Update system packages (optional but recommended):  
   `sudo apt update && sudo apt upgrade -y`
2. Install required tools and libraries:  
   `sudo apt install -y build-essential git perl libperl-dev liblwp-protocol-https-perl libfile-copy-recursive-perl libmarc-record-perl libxml-libxml-perl libxml-xpath-perl libdatetime-format-iso8601-perl cpanminus`  
   `sudo cpanm Archive::BagIt`
3. Clone SLUB SIP Builder repository:  
   `git clone https://git.slub-dresden.de/digital-preservation/SLUB_SIP_Builder.git`  
   `cd SLUB_SIP_Builder`  
   &rarr; Hint: For simple updates at a later date, just use `git pull` in directory `SLUB_SIP_Builder` to get the latest version.
4. Checkout the desired branch/tag/commit:  
   &rarr; for example most current master  
   `git checkout master`
5. Run unit tests (optional):  
   `perl t/slubsipbuilderbagit.t`
6. Run SLUB SIP Builder:  
   &rarr; Try an example command from below  
   `perl bin/slubsipbuilderbagit.pl`...

### Windows 10
1. Check UTF-8 flag at [ENG] `Systemadministration/TimeAndRegion/Region/Administration/AreaPattern` or [DE] `Systemsteuerung/Zeit und Region/Region/Verwaltung/Gebietsschema ändern/`, for using such characters as `äöüÄÖÜß`. Computer will be restarted!  
2. Download strawberry perl environment [strawberry-perl-5.32.1.1-64bit-portable](https://strawberryperl.com/releases.html)
3. Extract files from archive
4. Start strawberry perl console `portableshell.bat`
5. Install additional libraries:  
   `cpanm MARC::Record XML::XPath DateTime::Format::ISO8601 Win32::Console::ANSI Archive::BagIt`
6. Clone SLUB SIP Builder repository:  
   `git clone https://git.slub-dresden.de/digital-preservation/SLUB_SIP_Builder.git`  
   `cd SLUB_SIP_Builder`  
   &rarr; Hint: For simple updates at a later date, just use `git pull` in directory `SLUB_SIP_Builder` to get the latest version.
7. Run unit tests (optional):  
   `perl t\slubsipbuilderbagit.t`
8. Run SLUB SIP Builder:  
   &rarr; Try an example command from below  
   `perl bin\slubsipbuilderbagit.pl`...

## Building Releases

### Debian 11
1. Check external dependencies:  
   &rarr; i.e. Appropriate rights schema version included?
2. Build ZIP archive with release version:  
   `cd SLUB_SIP_Builder`  
   `bash build_release.sh`  
   &rarr; creates a ZIP file like`SLUB_SIP_Builder.version-XYZ-master.zip`
3. Create gitlab release
4. Add zip archive to gitlab release

## Documentation
* The Project is based on the BagIt 1.0 specification, visit [RFC8493](https://tools.ietf.org/html/rfc8493) for more information.
* For valid SIP examples based on operating systems, please have a look at `export_dir_kitodo/bagit/lin_*` or `export_dir_kitodo/bagit/win_*`
* Specification for creating a SLUBArchiv.digital SIP [SLUBArchiv_Produzenten_SIP_Spezifikation.pdf](https://slubarchiv.slub-dresden.de/technische-standards-fuer-die-ablieferung-von-digitalen-dokumenten/)
* For [valid SLUBArchiv.digital BagIt examples](https://git.slub-dresden.de/slub-digitalpreservation/submissionapplications4rosetta/-/tree/master/examples/bagit/valid_SLUB_bagits) and [faulty](https://git.slub-dresden.de/slub-digitalpreservation/submissionapplications4rosetta/-/tree/master/examples/bagit/faulty_SLUB_bagits) take a look at submission application project.
* SLUBArchiv.digital rights coding specification [SLUBArchiv_Produzenten_Rechteauszeichnung.pdf](https://slubarchiv.slub-dresden.de/technische-standards-fuer-die-ablieferung-von-digitalen-dokumenten/)

## Recommendation
* Do not name your directories `data`
* `marc21.xml` and `rights.xml` file names are reserved in directory `mets`. All same file names will be renamed to `1.xml`, `2.xml` and so on
* Use absolute file and directory paths
* If you try to execute the program for the first time, than use `save_option` copy

### Explanation
##### Usage
```perl
perl bin/slubsipbuilderbagit.pl -help
```
###### Result
```perl
Usage:
    slubsipbuilderbagit.pl [options]

    Options: -help brief help message -man full documentation

            -save_option=<option>           payload files save as <copy>, <move>, <replace>, the <copy> option should be prefered
            -IE_directory=<IE dir>          existing IE directory (absolute path!)
            -SIP_output_path=<target dir>   where to put the SIP dir (absolute path!)
            -ppn=<ppn>|-noppn=<noppn>       SWB-PPN or any identifier (uses minimalistic marc21)

            -external_workflow=<workflow>   mandatory, should be uniqe workflow name
            -external_id=<id>               mandatory, should be uniqe ID (i.e. a catalog ID), reuse an ID only when updating existing AIP
            -external_ISIL=<isil>           optional, ISIL number of library
            -external_conservation_flag     optional, if set no other "original" still exists
            -external_value_descr=<text>    mandatory, the reason why to archive
            -rights_version=<version>       mandatory, SLUB law managment specification, supported <1.0>

            -rights_xml=<file>              mandatory, path to SLUB rights xml file(absolute path!)
            -add_meta_file=<file>           optional, can be repeated, additional meta files(absolute path!)
            -add_key_value=<"key:value">    optional, can be repeated, additional key value pairs for bag-info.txt

Options:
    -help   Print a brief help message and exits.

```

##### Full documentation with examples
```perl
perl -I lib/ bin/slubsipbuilderbagit.pl -man
```
###### Result
* MUST: Write command in one line.
```perl
EXAMPLES
  Copy    (minimalistic)
    perl bin/slubsipbuilderbagit.pl --save_option=copy
    --IE_directory=/IE/git/SLUB_SIP_Builder/export_dir_kitodo/bagit/test2
    --SIP_output_path=/output_sips --ppn=457035137
    --external_id=10008 --external_workflow=kitodo --external_ISIL=DE-14
    --external_value_descr="Gesetzlicher Auftrag" --rights_version=1.0
    --rights_xml=/metadata/rights/Fallbeispiel-02.xml

  Copy
    perl bin/slubsipbuilderbagit.pl --save_option=copy
    --IE_directory=/IE/git/SLUB_SIP_Builder/export_dir_kitodo/bagit/test2
    --SIP_output_path=/output_sips --ppn=457035137
    --external_id=10008 --external_workflow=kitodo --external_ISIL=DE-14
    --external_value_descr="Gesetzlicher Auftrag" --rights_version=1.0
    --rights_xml=/metadata/rights/Fallbeispiel-01.xml
    --add_meta_file=/metadata/lido.xml --add_meta_file=/metadata/marc21.xml
    --add_key_value="Author:Lew Nikolajewitsch Tolstoi"
    --add_key_value="Titel:Krieg und Frieden" --add_key_value="Genre:Roman"

  Replace (minimalistic)
    NOT RECOMMENDED! IF FAILS, "IE" MUST BE MANUAL RESTORED TO THE PREVIOUS
    STATE!!!

    perl bin/slubsipbuilderbagit.pl --save_option=replace
    --IE_directory=/IE/SLUB_SIP_Builder/export_dir_kitodo/bagit/test2
    --ppn=457035137 --external_id=10008
    --external_workflow=kitodo --external_ISIL=DE-14
    --external_value_descr="Gesetzlicher Auftrag" --rights_version=1.0
    --rights_xml=/metadata/rights/Fallbeispiel-01.xml

  Move    (minimalistic)
    NOT RECOMMENDED! IF FAILS, "IE" MUST BE MANUAL RESTORED TO THE PREVIOUS
    STATE!!!

    perl bin/slubsipbuilderbagit.pl --save_option=move
    --IE_directory=/IE/git/SLUB_SIP_Builder/export_dir_kitodo/bagit/test2
    --SIP_output_path=/output_sips --ppn=457035137
    --external_id=10008 --external_workflow=kitodo --external_ISIL=DE-14
    --external_value_descr="Gesetzlicher Auftrag" --rights_version=1.0
    --rights_xml=/metadata/rights/Fallbeispiel-03.xml
```

##### Run tests
* Start script in project directory to run all tests properly.
* NOTE: while running tests path `tmp/bagit` will be created.
```perl
perl t/slubsipbuilderbagit.t
```
or
```perl
prove t
```
